@@ -5,34 +5,27 @@ import json
 import warnings
 from functools import cached_property
 from io import BytesIO
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Optional,
-    Union,
-)
-from typing_extensions import TypeGuard, assert_never
+from typing import TYPE_CHECKING, Any, TypeGuard
+from typing_extensions import assert_never
 from urllib.parse import parse_qs
 
-from django.conf import settings
-from django.core.files import uploadhandler
-from django.http.multipartparser import MultiPartParser
-
 from channels.db import database_sync_to_async
 from channels.generic.http import AsyncHttpConsumer
-from strawberry.http.async_base_view import AsyncBaseHTTPView, AsyncHTTPRequestAdapter
-from strawberry.http.exceptions import HTTPException
-from strawberry.http.sync_base_view import SyncBaseHTTPView, SyncHTTPRequestAdapter
+from django.conf import settings
+from django.core.files import uploadhandler
+from django.http.multipartparser import MultiPartParser
+from lia import AsyncHTTPRequestAdapter, FormData, HTTPException, SyncHTTPRequestAdapter
+
+from strawberry.http.async_base_view import AsyncBaseHTTPView
+from strawberry.http.sync_base_view import SyncBaseHTTPView
 from strawberry.http.temporal_response import TemporalResponse
-from strawberry.http.types import FormData
 from strawberry.http.typevars import Context, RootValue
 from strawberry.types.unset import UNSET
 
 from .base import ChannelsConsumer
 
 if TYPE_CHECKING:
-    from collections.abc import AsyncGenerator, Mapping
+    from collections.abc import AsyncGenerator, Callable, Mapping
 
     from strawberry.http import GraphQLHTTPResponse
     from strawberry.http.ides import GraphQL_IDE
@@ -84,7 +77,7 @@ class ChannelsRequest:
         return self.consumer.scope["method"].upper()
 
     @property
-    def content_type(self) -> Optional[str]:
+    def content_type(self) -> str | None:
         return self.headers.get("content-type", None)
 
     @cached_property
@@ -130,9 +123,31 @@ class BaseChannelsRequestAdapter:
         return self.request.headers
 
     @property
-    def content_type(self) -> Optional[str]:
+    def content_type(self) -> str | None:
         return self.request.content_type
 
+    @property
+    def url(self) -> str:
+        scheme = self.request.consumer.scope["scheme"]
+        host = self.headers.get("host", "localhost")
+        path = self.request.consumer.scope["path"]
+        query_string = self.request.consumer.scope["query_string"]
+        url = f"{scheme}://{host}{path}"
+        if query_string:
+            url += f"?{query_string.decode()}"
+        return url
+
+    @property
+    def cookies(self) -> Mapping[str, str]:
+        cookie_header = self.headers.get("cookie", "")
+        cookies = {}
+        if cookie_header:
+            for cookie in cookie_header.split(";"):
+                if "=" in cookie:
+                    key, value = cookie.split("=", 1)
+                    cookies[key.strip()] = value.strip()
+        return cookies
+
 
 class ChannelsRequestAdapter(BaseChannelsRequestAdapter, AsyncHTTPRequestAdapter):
     async def get_body(self) -> bytes:
@@ -148,23 +163,26 @@ class SyncChannelsRequestAdapter(BaseChannelsRequestAdapter, SyncHTTPRequestAdap
         return self.request.body
 
     @property
-    def post_data(self) -> Mapping[str, Union[str, bytes]]:
-        return self.request.form_data["form"]
+    def post_data(self) -> Mapping[str, str | bytes]:
+        return self.request.form_data.form
 
     @property
     def files(self) -> Mapping[str, Any]:
-        return self.request.form_data["files"]
+        return self.request.form_data.files
+
+    def get_form_data(self) -> FormData:
+        return self.request.form_data
 
 
 class BaseGraphQLHTTPConsumer(ChannelsConsumer, AsyncHttpConsumer):
     graphql_ide_html: str
-    graphql_ide: Optional[GraphQL_IDE] = "graphiql"
+    graphql_ide: GraphQL_IDE | None = "graphiql"
 
     def __init__(
         self,
         schema: BaseSchema,
-        graphiql: Optional[bool] = None,
-        graphql_ide: Optional[GraphQL_IDE] = "graphiql",
+        graphiql: bool | None = None,
+        graphql_ide: GraphQL_IDE | None = "graphiql",
         allow_queries_via_get: bool = True,
         multipart_uploads_enabled: bool = False,
         **kwargs: Any,
@@ -186,7 +204,9 @@ class BaseGraphQLHTTPConsumer(ChannelsConsumer, AsyncHttpConsumer):
         super().__init__(**kwargs)
 
     def create_response(
-        self, response_data: GraphQLHTTPResponse, sub_response: TemporalResponse
+        self,
+        response_data: GraphQLHTTPResponse | list[GraphQLHTTPResponse],
+        sub_response: TemporalResponse,
     ) -> ChannelsResponse:
         return ChannelsResponse(
             content=json.dumps(response_data).encode(),
@@ -227,7 +247,7 @@ class GraphQLHTTPConsumer(
     BaseGraphQLHTTPConsumer,
     AsyncBaseHTTPView[
         ChannelsRequest,
-        Union[ChannelsResponse, MultipartChannelsResponse],
+        ChannelsResponse | MultipartChannelsResponse,
         TemporalResponse,
         ChannelsRequest,
         TemporalResponse,
@@ -259,7 +279,7 @@ class GraphQLHTTPConsumer(
     allow_queries_via_get: bool = True
     request_adapter_class = ChannelsRequestAdapter
 
-    async def get_root_value(self, request: ChannelsRequest) -> Optional[RootValue]:
+    async def get_root_value(self, request: ChannelsRequest) -> RootValue | None:
         return None  # pragma: no cover
 
     async def get_context(
@@ -302,13 +322,11 @@ class GraphQLHTTPConsumer(
     ) -> TypeGuard[ChannelsRequest]:
         return False
 
-    async def pick_websocket_subprotocol(
-        self, request: ChannelsRequest
-    ) -> Optional[str]:
+    async def pick_websocket_subprotocol(self, request: ChannelsRequest) -> str | None:
         return None
 
     async def create_websocket_response(
-        self, request: ChannelsRequest, subprotocol: Optional[str]
+        self, request: ChannelsRequest, subprotocol: str | None
     ) -> TemporalResponse:
         raise NotImplementedError
 
@@ -333,7 +351,7 @@ class SyncGraphQLHTTPConsumer(
     allow_queries_via_get: bool = True
     request_adapter_class = SyncChannelsRequestAdapter
 
-    def get_root_value(self, request: ChannelsRequest) -> Optional[RootValue]:
+    def get_root_value(self, request: ChannelsRequest) -> RootValue | None:
         return None  # pragma: no cover
 
     def get_context(
@@ -361,7 +379,7 @@ class SyncGraphQLHTTPConsumer(
         self,
         request: ChannelsRequest,
         context: Context = UNSET,
-        root_value: Optional[RootValue] = UNSET,
+        root_value: RootValue | None = UNSET,
     ) -> ChannelsResponse | MultipartChannelsResponse:
         return super().run(request, context, root_value)
 
