@@ -8,11 +8,8 @@ from dataclasses import dataclass
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
     Generic,
-    Optional,
     TypeVar,
-    Union,
     overload,
 )
 
@@ -20,7 +17,14 @@ from .exceptions import WrongNumberOfResultsReturned
 
 if TYPE_CHECKING:
     from asyncio.events import AbstractEventLoop
-    from collections.abc import Awaitable, Hashable, Iterable, Mapping, Sequence
+    from collections.abc import (
+        Awaitable,
+        Callable,
+        Hashable,
+        Iterable,
+        Mapping,
+        Sequence,
+    )
 
 
 T = TypeVar("T")
@@ -46,9 +50,9 @@ class Batch(Generic[K, T]):
         return len(self.tasks)
 
 
-class AbstractCache(Generic[K, T], ABC):
+class AbstractCache(ABC, Generic[K, T]):
     @abstractmethod
-    def get(self, key: K) -> Union[Future[T], None]:
+    def get(self, key: K) -> Future[T] | None:
         pass
 
     @abstractmethod
@@ -65,13 +69,13 @@ class AbstractCache(Generic[K, T], ABC):
 
 
 class DefaultCache(AbstractCache[K, T]):
-    def __init__(self, cache_key_fn: Optional[Callable[[K], Hashable]] = None) -> None:
+    def __init__(self, cache_key_fn: Callable[[K], Hashable] | None = None) -> None:
         self.cache_key_fn: Callable[[K], Hashable] = (
             cache_key_fn if cache_key_fn is not None else lambda x: x
         )
         self.cache_map: dict[Hashable, Future[T]] = {}
 
-    def get(self, key: K) -> Union[Future[T], None]:
+    def get(self, key: K) -> Future[T] | None:
         return self.cache_map.get(self.cache_key_fn(key))
 
     def set(self, key: K, value: Future[T]) -> None:
@@ -85,7 +89,7 @@ class DefaultCache(AbstractCache[K, T]):
 
 
 class DataLoader(Generic[K, T]):
-    batch: Optional[Batch[K, T]] = None
+    batch: Batch[K, T] | None = None
     cache: bool = False
     cache_map: AbstractCache[K, T]
 
@@ -93,12 +97,12 @@ class DataLoader(Generic[K, T]):
     def __init__(
         self,
         # any BaseException is rethrown in 'load', so should be excluded from the T type
-        load_fn: Callable[[list[K]], Awaitable[Sequence[Union[T, BaseException]]]],
-        max_batch_size: Optional[int] = None,
+        load_fn: Callable[[list[K]], Awaitable[Sequence[T | BaseException]]],
+        max_batch_size: int | None = None,
         cache: bool = True,
-        loop: Optional[AbstractEventLoop] = None,
-        cache_map: Optional[AbstractCache[K, T]] = None,
-        cache_key_fn: Optional[Callable[[K], Hashable]] = None,
+        loop: AbstractEventLoop | None = None,
+        cache_map: AbstractCache[K, T] | None = None,
+        cache_key_fn: Callable[[K], Hashable] | None = None,
     ) -> None: ...
 
     # fallback if load_fn is untyped and there's no other info for inference
@@ -106,21 +110,21 @@ class DataLoader(Generic[K, T]):
     def __init__(
         self: DataLoader[K, Any],
         load_fn: Callable[[list[K]], Awaitable[list[Any]]],
-        max_batch_size: Optional[int] = None,
+        max_batch_size: int | None = None,
         cache: bool = True,
-        loop: Optional[AbstractEventLoop] = None,
-        cache_map: Optional[AbstractCache[K, T]] = None,
-        cache_key_fn: Optional[Callable[[K], Hashable]] = None,
+        loop: AbstractEventLoop | None = None,
+        cache_map: AbstractCache[K, T] | None = None,
+        cache_key_fn: Callable[[K], Hashable] | None = None,
     ) -> None: ...
 
     def __init__(
         self,
-        load_fn: Callable[[list[K]], Awaitable[Sequence[Union[T, BaseException]]]],
-        max_batch_size: Optional[int] = None,
+        load_fn: Callable[[list[K]], Awaitable[Sequence[T | BaseException]]],
+        max_batch_size: int | None = None,
         cache: bool = True,
-        loop: Optional[AbstractEventLoop] = None,
-        cache_map: Optional[AbstractCache[K, T]] = None,
-        cache_key_fn: Optional[Callable[[K], Hashable]] = None,
+        loop: AbstractEventLoop | None = None,
+        cache_map: AbstractCache[K, T] | None = None,
+        cache_key_fn: Callable[[K], Hashable] | None = None,
     ):
         self.load_fn = load_fn
         self.max_batch_size = max_batch_size
@@ -244,7 +248,7 @@ async def dispatch_batch(loader: DataLoader, batch: Batch) -> None:
                 expected=len(batch), received=len(values)
             )
 
-        for task, value in zip(batch.tasks, values):
+        for task, value in zip(batch.tasks, values, strict=True):
             # Trying to set_result in a cancelled future would raise
             # asyncio.exceptions.InvalidStateError
             if task.future.cancelled():
