@@ -5,10 +5,7 @@ from itertools import chain
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
-    Optional,
     TypeVar,
-    Union,
     cast,
     overload,
 )
@@ -46,6 +43,8 @@ from strawberry.types.unset import UNSET
 from .ast_from_value import ast_from_value
 
 if TYPE_CHECKING:
+    from collections.abc import Callable
+
     from graphql import (
         GraphQLArgument,
         GraphQLEnumType,
@@ -78,7 +77,7 @@ def _serialize_dataclasses(
 
 @overload
 def _serialize_dataclasses(
-    value: Union[list[object], tuple[object]],
+    value: list[object] | tuple[object],
     *,
     name_converter: Callable[[str], str] | None = None,
 ) -> list[object]: ...
@@ -190,7 +189,7 @@ def print_schema_directive(
 
 
 def print_field_directives(
-    field: Optional[StrawberryField], schema: BaseSchema, *, extras: PrintExtras
+    field: StrawberryField | None, schema: BaseSchema, *, extras: PrintExtras
 ) -> str:
     if not field:
         return ""
@@ -363,7 +362,7 @@ def print_extends(type_: GraphQLObjectType, schema: BaseSchema) -> str:
     from strawberry.schema.schema_converter import GraphQLCoreConverter
 
     strawberry_type = cast(
-        "Optional[StrawberryObjectDefinition]",
+        "StrawberryObjectDefinition | None",
         type_.extensions
         and type_.extensions.get(GraphQLCoreConverter.DEFINITION_BACKREF),
     )
@@ -380,7 +379,7 @@ def print_type_directives(
     from strawberry.schema.schema_converter import GraphQLCoreConverter
 
     strawberry_type = cast(
-        "Optional[StrawberryObjectDefinition]",
+        "StrawberryObjectDefinition | None",
         type_.extensions
         and type_.extensions.get(GraphQLCoreConverter.DEFINITION_BACKREF),
     )
@@ -534,9 +533,7 @@ def _all_root_names_are_common_names(schema: BaseSchema) -> bool:
     )
 
 
-def print_schema_definition(
-    schema: BaseSchema, *, extras: PrintExtras
-) -> Optional[str]:
+def print_schema_definition(schema: BaseSchema, *, extras: PrintExtras) -> str | None:
     # TODO: add support for description
 
     if _all_root_names_are_common_names(schema) and not schema.schema_directives:
@@ -558,12 +555,10 @@ def print_schema_definition(
     return f"schema{directives} {{\n" + "\n".join(operation_types) + "\n}"
 
 
-def print_directive(
-    directive: GraphQLDirective, *, schema: BaseSchema
-) -> Optional[str]:
-    strawberry_directive = directive.extensions["strawberry-definition"]
+def print_directive(directive: GraphQLDirective, *, schema: BaseSchema) -> str | None:
+    strawberry_directive = directive.extensions.get("strawberry-definition")
 
-    if (
+    if strawberry_directive is None or (
         isinstance(strawberry_directive, StrawberrySchemaDirective)
         and not strawberry_directive.print_definition
     ):
@@ -621,6 +616,14 @@ def print_schema(schema: BaseSchema) -> str:
         if (printed_directive := print_directive(directive, schema=schema)) is not None
     ]
 
+    if schema.config.enable_experimental_incremental_execution:
+        directives.append(
+            "directive @defer(if: Boolean, label: String) on FRAGMENT_SPREAD | INLINE_FRAGMENT"
+        )
+        directives.append(
+            "directive @stream(if: Boolean, label: String, initialCount: Int = 0) on FIELD"
+        )
+
     def _name_getter(type_: Any) -> str:
         if hasattr(type_, "name"):
             return type_.name
