@@ -2,15 +2,14 @@ from __future__ import annotations
 
 import dataclasses
 import sys
+import typing
 from functools import partial, reduce
 from typing import (
     TYPE_CHECKING,
+    Annotated,
     Any,
-    Callable,
     Generic,
-    Optional,
     TypeVar,
-    Union,
     cast,
 )
 from typing_extensions import Protocol
@@ -80,7 +79,7 @@ from . import compat
 from .types.concrete_type import ConcreteType
 
 if TYPE_CHECKING:
-    from collections.abc import Awaitable, Mapping
+    from collections.abc import Awaitable, Callable, Mapping
 
     from graphql import (
         GraphQLInputType,
@@ -100,17 +99,17 @@ if TYPE_CHECKING:
 
 FieldType = TypeVar(
     "FieldType",
-    bound=Union[GraphQLField, GraphQLInputField],
+    bound=GraphQLField | GraphQLInputField,
     covariant=True,
 )
 
 
-class FieldConverterProtocol(Generic[FieldType], Protocol):
+class FieldConverterProtocol(Protocol, Generic[FieldType]):
     def __call__(  # pragma: no cover
         self,
         field: StrawberryField,
         *,
-        type_definition: Optional[StrawberryObjectDefinition] = None,
+        type_definition: StrawberryObjectDefinition | None = None,
     ) -> FieldType: ...
 
 
@@ -180,7 +179,7 @@ class CustomGraphQLEnumType(GraphQLEnumType):
         return self.wrapped_cls(super().parse_value(input_value))
 
     def parse_literal(
-        self, value_node: ValueNode, _variables: Optional[dict[str, Any]] = None
+        self, value_node: ValueNode, _variables: dict[str, Any] | None = None
     ) -> Any:
         return self.wrapped_cls(super().parse_literal(value_node, _variables))
 
@@ -192,7 +191,7 @@ def get_arguments(
     info: Info,
     kwargs: Any,
     config: StrawberryConfig,
-    scalar_registry: Mapping[object, Union[ScalarWrapper, ScalarDefinition]],
+    scalar_registry: Mapping[object, ScalarWrapper | ScalarDefinition],
 ) -> tuple[list[Any], dict[str, Any]]:
     # TODO: An extension might have changed the resolver arguments,
     # but we need them here since we are calling it.
@@ -249,7 +248,7 @@ class GraphQLCoreConverter:
     def __init__(
         self,
         config: StrawberryConfig,
-        scalar_overrides: Mapping[object, Union[ScalarWrapper, ScalarDefinition]],
+        scalar_overrides: Mapping[object, ScalarWrapper | ScalarDefinition],
         get_fields: Callable[[StrawberryObjectDefinition], list[StrawberryField]],
     ) -> None:
         self.type_map: dict[str, ConcreteType] = {}
@@ -259,8 +258,8 @@ class GraphQLCoreConverter:
 
     def _get_scalar_registry(
         self,
-        scalar_overrides: Mapping[object, Union[ScalarWrapper, ScalarDefinition]],
-    ) -> Mapping[object, Union[ScalarWrapper, ScalarDefinition]]:
+        scalar_overrides: Mapping[object, ScalarWrapper | ScalarDefinition],
+    ) -> Mapping[object, ScalarWrapper | ScalarDefinition]:
         scalar_registry = {**DEFAULT_SCALAR_REGISTRY}
 
         global_id_name = "GlobalID" if self.config.relay_use_legacy_global_id else "ID"
@@ -407,7 +406,7 @@ class GraphQLCoreConverter:
         self,
         field: StrawberryField,
         *,
-        type_definition: Optional[StrawberryObjectDefinition] = None,
+        type_definition: StrawberryObjectDefinition | None = None,
     ) -> GraphQLField:
         # self.from_resolver needs to be called before accessing field.type because
         # in there a field extension might want to change the type during its apply
@@ -445,7 +444,7 @@ class GraphQLCoreConverter:
         self,
         field: StrawberryField,
         *,
-        type_definition: Optional[StrawberryObjectDefinition] = None,
+        type_definition: StrawberryObjectDefinition | None = None,
     ) -> GraphQLInputField:
         field_type = cast(
             "GraphQLInputType",
@@ -556,14 +555,14 @@ class GraphQLCoreConverter:
 
         def _get_resolve_type() -> Callable[
             [Any, GraphQLResolveInfo, GraphQLAbstractType],
-            Union[Awaitable[Optional[str]], str, None],
+            Awaitable[str | None] | str | None,
         ]:
             if interface.resolve_type:
                 return interface.resolve_type
 
             def resolve_type(
                 obj: Any, info: GraphQLResolveInfo, abstract_type: GraphQLAbstractType
-            ) -> Union[Awaitable[Optional[str]], str, None]:
+            ) -> Awaitable[str | None] | str | None:
                 if isinstance(obj, interface.origin):
                     type_definition = get_object_definition(obj, strict=True)
 
@@ -578,7 +577,7 @@ class GraphQLCoreConverter:
                     # all the types in the schema, but we should probably
                     # optimize this
 
-                    return_type: Optional[GraphQLType] = None
+                    return_type: GraphQLType | None = None
 
                     for possible_concrete_type in self.type_map.values():
                         possible_type = possible_concrete_type.definition
@@ -639,7 +638,7 @@ class GraphQLCoreConverter:
             assert isinstance(graphql_object_type, GraphQLObjectType)  # For mypy
             return graphql_object_type
 
-        def _get_is_type_of() -> Optional[Callable[[Any, GraphQLResolveInfo], bool]]:
+        def _get_is_type_of() -> Callable[[Any, GraphQLResolveInfo], bool] | None:
             if object_type.is_type_of:
                 return object_type.is_type_of
 
@@ -849,19 +848,32 @@ class GraphQLCoreConverter:
         return implementation
 
     def from_maybe_optional(
-        self, type_: Union[StrawberryType, type]
-    ) -> Union[GraphQLNullableType, GraphQLNonNull]:
+        self, type_: StrawberryType | type
+    ) -> GraphQLNullableType | GraphQLNonNull:
         NoneType = type(None)
         if type_ is None or type_ is NoneType:
             return self.from_type(type_)
+        if isinstance(type_, StrawberryMaybe):
+            # StrawberryMaybe should always generate optional types
+            # because Maybe[T] = Union[Some[T], None] (field can be absent)
+            # But we need to handle the case where of_type is itself optional
+            if isinstance(type_.of_type, StrawberryOptional):
+                return self.from_type(type_.of_type.of_type)
+            return self.from_type(type_.of_type)
         if isinstance(type_, StrawberryOptional):
             return self.from_type(type_.of_type)
         return GraphQLNonNull(self.from_type(type_))
 
-    def from_type(self, type_: Union[StrawberryType, type]) -> GraphQLNullableType:
+    def from_type(self, type_: StrawberryType | type) -> GraphQLNullableType:
         if compat.is_graphql_generic(type_):
             raise MissingTypesForGenericError(type_)
 
+        # to handle lazy unions
+        if typing.get_origin(type_) is Annotated:
+            args = typing.get_args(type_)
+            if len(args) >= 2 and isinstance(args[1], StrawberryUnion):
+                type_ = args[1]
+
         if isinstance(type_, EnumDefinition):  # TODO: Replace with StrawberryEnum
             return self.from_enum(type_)
         if compat.is_input_type(type_):  # TODO: Replace with StrawberryInputObject
@@ -945,7 +957,7 @@ class GraphQLCoreConverter:
     def _get_is_type_of(
         self,
         object_type: StrawberryObjectDefinition,
-    ) -> Optional[Callable[[Any, GraphQLResolveInfo], bool]]:
+    ) -> Callable[[Any, GraphQLResolveInfo], bool] | None:
         if object_type.is_type_of:
             return object_type.is_type_of
 
@@ -1003,43 +1015,11 @@ class GraphQLCoreConverter:
         first_type_definition = cached_type.definition
         second_type_definition = type_definition
 
-        # TODO: maybe move this on the StrawberryType class
-        if (
-            isinstance(first_type_definition, StrawberryObjectDefinition)
-            and isinstance(second_type_definition, StrawberryObjectDefinition)
-            and first_type_definition.concrete_of is not None
-            and first_type_definition.concrete_of == second_type_definition.concrete_of
-            and (
-                first_type_definition.type_var_map.keys()
-                == second_type_definition.type_var_map.keys()
-            )
+        if self.is_same_type_definition(
+            first_type_definition,
+            second_type_definition,
         ):
-            # manually compare type_var_maps while resolving any lazy types
-            # so that they're considered equal to the actual types they're referencing
-            equal = True
-            for type_var, type1 in first_type_definition.type_var_map.items():
-                type2 = second_type_definition.type_var_map[type_var]
-                # both lazy types are always resolved because two different lazy types
-                # may be referencing the same actual type
-                if isinstance(type1, LazyType):
-                    type1 = type1.resolve_type()  # noqa: PLW2901
-                elif isinstance(type1, StrawberryOptional) and isinstance(
-                    type1.of_type, LazyType
-                ):
-                    type1.of_type = type1.of_type.resolve_type()
-
-                if isinstance(type2, LazyType):
-                    type2 = type2.resolve_type()
-                elif isinstance(type2, StrawberryOptional) and isinstance(
-                    type2.of_type, LazyType
-                ):
-                    type2.of_type = type2.of_type.resolve_type()
-
-                if type1 != type2:
-                    equal = False
-                    break
-            if equal:
-                return
+            return
 
         if isinstance(second_type_definition, StrawberryObjectDefinition):
             first_origin = second_type_definition.origin
@@ -1057,5 +1037,63 @@ class GraphQLCoreConverter:
 
         raise DuplicatedTypeName(first_origin, second_origin, name)
 
+    def is_same_type_definition(
+        self,
+        first_type_definition: StrawberryObjectDefinition | StrawberryType,
+        second_type_definition: StrawberryObjectDefinition | StrawberryType,
+    ) -> bool:
+        # TODO: maybe move this on the StrawberryType class
+        if (
+            not isinstance(first_type_definition, StrawberryObjectDefinition)
+            or not isinstance(second_type_definition, StrawberryObjectDefinition)
+            or first_type_definition.concrete_of is None
+            or first_type_definition.concrete_of != second_type_definition.concrete_of
+            or (
+                first_type_definition.type_var_map.keys()
+                != second_type_definition.type_var_map.keys()
+            )
+        ):
+            return False
+
+        # manually compare type_var_maps while resolving any lazy types
+        # so that they're considered equal to the actual types they're referencing
+        for type_var, type1 in first_type_definition.type_var_map.items():
+            type2 = second_type_definition.type_var_map[type_var]
+
+            # both lazy types are always resolved because two different lazy types
+            # may be referencing the same actual type
+            if isinstance(type1, LazyType):
+                type1 = type1.resolve_type()  # noqa: PLW2901
+            elif isinstance(type1, StrawberryOptional) and isinstance(
+                type1.of_type, LazyType
+            ):
+                type1.of_type = type1.of_type.resolve_type()
+
+            if isinstance(type2, LazyType):
+                type2 = type2.resolve_type()
+            elif isinstance(type2, StrawberryOptional) and isinstance(
+                type2.of_type, LazyType
+            ):
+                type2.of_type = type2.of_type.resolve_type()
+
+            same_type = type1 == type2
+            # If both types have object definitions, we are handling a nested generic
+            # type like `Foo[Foo[int]]`, meaning we need to compare their type definitions
+            # as they will actually be different instances of the type
+            if (
+                not same_type
+                and has_object_definition(type1)
+                and has_object_definition(type2)
+            ):
+                same_type = self.is_same_type_definition(
+                    type1.__strawberry_definition__,
+                    type2.__strawberry_definition__,
+                )
+
+            if not same_type:
+                return False
+
+        return True
+
 
 __all__ = ["GraphQLCoreConverter"]
