@@ -1,6 +1,6 @@
+import re
 import typing
-from typing import TYPE_CHECKING, Any, Generic, TypeVar, Union
-from typing_extensions import TypeAlias
+from typing import TYPE_CHECKING, Any, Generic, TypeAlias, TypeVar
 
 T = TypeVar("T")
 
@@ -31,13 +31,22 @@ class Some(Generic[T]):
 
 
 if TYPE_CHECKING:
-    Maybe: TypeAlias = Union[Some[Union[T, None]], None]
+    Maybe: TypeAlias = Some[T] | None
 else:
     # we do this trick so we can inspect that at runtime
     class Maybe(Generic[T]): ...
 
 
+_maybe_re = re.compile(r"^(?:strawberry\.)?Maybe\[(.+)\]$")
+
+
 def _annotation_is_maybe(annotation: Any) -> bool:
+    if isinstance(annotation, str):
+        # Ideally we would try to evaluate the annotation, but the args inside
+        # may still not be available, as the module is still being constructed.
+        # Checking for the pattern should be good enough for now.
+        return _maybe_re.match(annotation) is not None
+
     return (orig := typing.get_origin(annotation)) and orig is Maybe
 
 
