@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import warnings
-from typing import Any, Optional
+from typing import Any
 
 
 class DEPRECATION_MESSAGES:  # noqa: N801
@@ -19,7 +19,7 @@ class DeprecatedDescriptor:
     def warn(self) -> None:
         warnings.warn(self.msg, stacklevel=2)
 
-    def __get__(self, obj: Optional[object], type: Optional[type] = None) -> Any:
+    def __get__(self, obj: object | None, type: type | None = None) -> Any:
         self.warn()
         return self.alias
 
