@@ -3,7 +3,7 @@ from __future__ import annotations
 import asyncio
 import json
 from collections.abc import AsyncGenerator
-from typing import TYPE_CHECKING, Union
+from typing import TYPE_CHECKING
 from unittest import mock
 
 import pytest
@@ -20,7 +20,7 @@ from strawberry.subscriptions.protocols.graphql_ws.types import (
     ErrorMessage,
     StartMessage,
 )
-from tests.views.schema import MyExtension, Schema, Subscription
+from tests.views.schema import MyExtension, Schema, Subscription, schema
 
 if TYPE_CHECKING:
     from tests.http.clients.base import HttpClient, WebSocketClient
@@ -290,7 +290,7 @@ async def test_context_can_be_modified_from_within_on_ws_connect(
 
 
 async def test_sends_keep_alive(http_client_class: type[HttpClient]):
-    http_client = http_client_class(keep_alive=True, keep_alive_interval=0.1)
+    http_client = http_client_class(schema, keep_alive=True, keep_alive_interval=0.1)
 
     async with http_client.ws_connect(
         "/graphql", protocols=[GRAPHQL_WS_PROTOCOL]
@@ -313,9 +313,9 @@ async def test_sends_keep_alive(http_client_class: type[HttpClient]):
         # get but they should be more than one.
         keepalive_count = 0
         while True:
-            ka_or_data_message: Union[
-                ConnectionKeepAliveMessage, DataMessage
-            ] = await ws.receive_json()
+            ka_or_data_message: (
+                ConnectionKeepAliveMessage | DataMessage
+            ) = await ws.receive_json()
             if ka_or_data_message["type"] == "ka":
                 keepalive_count += 1
             else:
